/*  ============================================================================
 *   Author: Tyler Olivieri 
 *   Description: Setting up the DMA in Ping Pong mode by writing directly to registers, abstracted away with obj oriented principles. 
 * 	 Added driver to do DARAM to DARAM transfers.
 * 	
 *   Added driver to do DARAM to DARAM transfers 
 * 
 * 	 Reference -> Nathan Zorndorf 
 */
 
#include <soc.h>
#include <csl_intc.h>
#include <stdio.h>
#include <usbstk5505.h>
#include <My_I2S_Register.h>
#include <Application_1_Modified_Registers.h>
#include <My_DMA_Ping_Pong_Register_Setup.h>
#include "dma_driver.h"

/* Interrupt vector start address */  
extern void VECSTART(void);	 			// WHERE IS THIS DEFINED/DECLARED 

#define transferLength00 WND_LEN
#define transferLength10 PING_PONG_SIZE

	Dma_Obj DMA0CH0;
	Dma_Obj DMA0CH1;
	Dma_Obj DMA0CH2;
	Dma_Obj DMA0CH3;
	Dma_Obj DMA1CH0;
	Dma_Obj DMA1CH1;
	Dma_Obj DMA1CH2;
	Dma_Obj DMA1CH3;
	
 int My_DMA_Ping_Pong_Register_Setup(void) {
 	
 	Uint16 register_value;
	Uint32 dma_address; 
	Int16  testSourceL[transferLength00];
	Int16  testSourceR[transferLength00];  
	Int16 testDestL[transferLength00]; 
	Int16 testDestR[transferLength00]; 
	Int16  testFinishL[transferLength00];
	Int16  testFinishR[transferLength00];
	unsigned int i;
	Int16 status; 
	
	memset(&testSourceL, 10000, sizeof(Int16)*transferLength00);
    memset(&testSourceR, 20000, sizeof(Int16)*transferLength00);
	memset(&testDestL, 10, sizeof(Int16)*transferLength00);
	memset(&testDestR, 20, sizeof(Int16)*transferLength00);
	memset(&testFinishL, 100, sizeof(Int16)*transferLength00);
    memset(&testFinishR, 200, sizeof(Int16)*transferLength00);

	
     // --------------------- Interrupts Setup --------------------- /  
      //printf("DMA INTERRUPTS SETUP BEGIN!!\n");
     	//IRQ_globalDisable(); // It disables the interrupt globally by disabling INTM bit and also return the previous mask value for INTM bit
	 IRQ_clearAll(); // This function clears all the interrupts. Both IFR0 and IFR1 are cleared by this function.
	 	//IRQ_disableAll(); // This function disables all the interrupts avaible on C5505 DSP. Both IER0 and IER1 are cleared by this function
	 IRQ_setVecs((Uint32)(&VECSTART)); // It stores the Interrupt vector table address in Interrupt vector pointer DSP and Interrupt vector pointer host Registers
         	 //IRQ_clear(DMA_EVENT); // possibly not necessary and might need to go before(?) or after(?) IRQ_plug()
 	 IRQ_plug(DMA_EVENT, &DMA_ISR); // This function is used to register the ISR routine for the corresponding interrupt event.
     IRQ_enable(DMA_EVENT); // It enables the corresponding interrupt bit in IER Register and also return the previous bit mask value
	 IRQ_globalEnable(); // It enables the interrupt globally by enabling INTM bit and also return the previous mask value for INTM bit
	 //printf("DMA INTERRUPTS SETUP END!!\n");
	 // ------------------------------------------------------------ /
 	
 	
 	//printf("DMA SETUP BEGIN!!\n");
 	
	status = Dma_init((Dma_Obj *) &DMA0CH0,  (Uint16)0x0C00, DARAM2DARAM);
	status = Dma_interrupts();		 
	status = Dma_Config((Dma_Obj *) &DMA0CH0, testSourceL, testDestL,  transferLength00);
	
	status = Dma_init((Dma_Obj *) &DMA0CH1,  (Uint16)0x0C20, DARAM2DARAM);
	status = Dma_interrupts();		 
	status = Dma_Config((Dma_Obj *) &DMA0CH1, testSourceR, testDestR,  transferLength00);
	
	status = Dma_start((Dma_Obj *) &DMA0CH0);
	status = Dma_start((Dma_Obj *) &DMA0CH1);
	
	status = Dma_init((Dma_Obj *) &DMA0CH2,  (Uint16)0x0C40, DARAM2DARAM);
	status = Dma_interrupts();		 
	status = Dma_Config((Dma_Obj *) &DMA0CH2, testDestL, testFinishL,  transferLength00);
	
	status = Dma_init((Dma_Obj *) &DMA0CH3,  (Uint16)0x0C60, DARAM2DARAM);
	status = Dma_interrupts();		 
	status = Dma_Config((Dma_Obj *) &DMA0CH3, testDestR, testFinishR,  transferLength00);
	
 	
 	/*        
    //memmove(&BufferL[0],&OverlapInL[0], sizeof(Int16)*WND_LEN); memmove(&BufferR[0],&OverlapInR[0], sizeof(Int16)*WND_LEN);
	status = Dma_init((Dma_Obj *) &DMA0CH0,  (Uint16)0x0C00, DARAM2DARAM);
	status = Dma_interrupts();		 
	status = Dma_Config((Dma_Obj *) &DMA0CH0, OverlapInL, BufferL,  transferLength00);

    status = Dma_init((Dma_Obj *) &DMA0CH1,  (Uint16)0x0C20, DARAM2DARAM);	 
	status = Dma_Config((Dma_Obj *) &DMA0CH1, OverlapInR, BufferR,  transferLength00);
    
    //DMA_OutL[i + AUDIO_IO_SIZE] = BufferL[i];DMA_OutR[i + AUDIO_IO_SIZE] = BufferR[i]; 
    status = Dma_init((Dma_Obj *) &DMA0CH2,  (Uint16)0x0C40, DARAM2DARAM);
	status = Dma_Config((Dma_Obj *) &DMA0CH2, OverlapInL, BufferL,  transferLength00);
    
    status = Dma_init((Dma_Obj *) &DMA0CH3,  (Uint16)0x0C60, DARAM2DARAM);	 
	status = Dma_Config((Dma_Obj *) &DMA0CH3, OverlapInL, BufferL,  transferLength00);
	
	//OverlapOutL[i] = BufferL[i + FRAME]; OverlapOutR[i] = BufferR[i + FRAME];
	//status = Dma_init((Dma_Obj *) &DMA2CH0,  (Uint16)0x0C00, DARAM2DARAM);	 
	//status = Dma_Config((Dma_Obj *) &DMA0CH0, OverlapInL, BufferL,  transferLength00);

    //status = Dma_init((Dma_Obj *) &DMA2CH1,  (Uint16)0x0C20, DARAM2DARAM);	 
	//status = Dma_Config((Dma_Obj *) &DMA0CH1, OverlapInR, BufferR,  transferLength00);
    
    //DMA_OutL[i] =  BufferL[i];DMA_OutR[i] =  BufferR[i]; 
    status = Dma_init((Dma_Obj *) &DMA2CH2,  (Uint16)0x0C40, DARAM2DARAM);
	status = Dma_Config((Dma_Obj *) &DMA2CH2, OverlapInL, BufferL,  transferLength00);
    
    status = Dma_init((Dma_Obj *) &DMA2CH3,  (Uint16)0x0C60, DARAM2DARAM);	 
	status = Dma_Config((Dma_Obj *) &DMA2CH3, OverlapInL, BufferL,  transferLength00);
	
    //memmove(&OverlapInL[FRAME],&DMA_InpL[AUDIO_IO_SIZE], sizeof(Int16)*AUDIO_IO_SIZE);
    //memmove(&OverlapInR[FRAME],&DMA_InpR[AUDIO_IO_SIZE], sizeof(Int16)*AUDIO_IO_SIZE); 
    status = Dma_init((Dma_Obj *) &DMA3CH0,  (Uint16)0x0C60, DARAM2DARAM);	 
	status = Dma_Config((Dma_Obj *) &DMA3CH0, OverlapInL, BufferL,  transferLength00);
	
	status = Dma_init((Dma_Obj *) &DMA3CH1,  (Uint16)0x0C60, DARAM2DARAM);	 
	status = Dma_Config((Dma_Obj *) &DMA3CH1, OverlapInL, BufferL,  transferLength00);
  
  	//memmove(&OverlapInL[0],&OverlapInL[FRAME], sizeof(Int16)*FRAME);memmove(&OverlapInR[0],&OverlapInR[FRAME], sizeof(Int16)*FRAME);
  	status = Dma_init((Dma_Obj *) &DMA3CH2,  (Uint16)0x0C60, DARAM2DARAM);	 
	status = Dma_Config((Dma_Obj *) &DMA3CH2, OverlapInL, BufferL,  transferLength00);
	
	status = Dma_init((Dma_Obj *) &DMA3CH3,  (Uint16)0x0C60, DARAM2DARAM);	 
	status = Dma_Config((Dma_Obj *) &DMA3CH3, OverlapInL, BufferL,  transferLength00);
	*/
    //codec to DARAM
    //DARAM to codec
    status = Dma_init((Dma_Obj *) &DMA1CH0,  (Uint16)0x0D00, destDARAM);
    status = Dma_init((Dma_Obj *) &DMA1CH1,  (Uint16)0x0D20, destDARAM);
    status = Dma_init((Dma_Obj *) &DMA1CH2,  (Uint16)0x0D40, sourceDARAM);
    status = Dma_init((Dma_Obj *) &DMA1CH3,  (Uint16)0x0D60, sourceDARAM);
    
    status = Dma_Config((Dma_Obj *) &DMA1CH0,(Uint16*)0x2A28, DMA_InpL,  transferLength10);
    status = Dma_Config((Dma_Obj *) &DMA1CH1,(Uint16*)0x2A2C, DMA_InpR,  transferLength10);
    status = Dma_Config((Dma_Obj *) &DMA1CH2, DMA_OutL, (Uint16*)0x2A08,  transferLength10);
    status = Dma_Config((Dma_Obj *) &DMA1CH3, DMA_OutR, (Uint16*)0x2A0C,  transferLength10);
    
    status = Dma_enablePingPong ((Dma_Obj *) &DMA1CH0);
    status = Dma_enablePingPong ((Dma_Obj *) &DMA1CH1);
    status = Dma_enablePingPong ((Dma_Obj *) &DMA1CH2);
    status = Dma_enablePingPong ((Dma_Obj *) &DMA1CH3);
    
    status = Dma_enable_I2S();
    
    status = Dma_start((Dma_Obj *) &DMA1CH0);
    status = Dma_start((Dma_Obj *) &DMA1CH1);
    status = Dma_start((Dma_Obj *) &DMA1CH2);
    status = Dma_start((Dma_Obj *) &DMA1CH3);
    
 		 
 	return(0);
 	
 }

 
 interrupt void DMA_ISR(void)
{
	Uint16 register_value1, register_value2;
	
	/* Clear CPU DMA interrupt */
	register_value1 = IFR0;
	IFR0 = register_value1;
	   
	/* Read DMA interrupt flags */
    register_value1 = DMA_IFR;
    
    //DMA0 Channel 0-1 interrupts are flagged, overlap transfers are complete, set flag, clear interrupt, start next transfer
	if(register_value1 & 0x0003) {
		straightThroughFlag = 1;
		DMA_IFR = 0x0003;
		Dma_start((Dma_Obj *) &DMA0CH2);
		Dma_start((Dma_Obj *) &DMA0CH3);
	} else{
	    straightThroughFlag = 0;	
	}
	
	if(register_value1 & 0x000C){ //DMA0 Channel  2-3 interrupts are flagged, set flag, clear interrupt
		doneFlag = 1;
		DMA_IFR = 0x000C;
	}
	else{
		doneFlag = 0;
	}
    /* DMA1 Channels 0-1, input */
    if (register_value1 & 0x0030) // if DMA 1 channel 0 and 1 interrupts are flagged  
    {
    	register_value2 = DMA1_CH0_TCR2;
    	if (register_value2 & 0x0002) { 
    		PingPongFlagInL = 1; // Last Transfer complete was Pong - Filling Ping
    	} else {
    		PingPongFlagInL = 0; // Last Transfer complete was Ping - Filling Pong
    	}
    	
    	register_value2 = DMA1_CH1_TCR2;
    	if (register_value2 & 0x0002) {
    		PingPongFlagInR = 1; // Last Transfer complete was Pong - Filling Ping
    	} else {
    		PingPongFlagInR = 0; // Last Transfer complete was Ping - Filling Pong
    	}
    		    
    	/* Clear CH0-1 interrupts */
    	DMA_IFR = 0x0030; 
    }
    
    /* DMA1 Channels 2-3, output */
    if (register_value1 & 0x00C0)  // if DMA 1 channel 2-3 interrupts are flagged 
    { 
    	register_value2 = DMA1_CH2_TCR2;
    	if (register_value2 & 0x0002) {
    		PingPongFlagOutL = 1;
    	} else {
    		PingPongFlagOutL = 0;
    	}
    	
    	register_value2 = DMA1_CH3_TCR2;
    	if (register_value2 & 0x0002) {
    		PingPongFlagOutR = 1;
    	} else {
    		PingPongFlagOutR = 0;
    	}
    	  	
    	/* Clear CH2-3 interrupts */
    	DMA_IFR = 0x00C0; 
    }
    
  /*  if(straightThroughFlag == 1){
    	if (PingPongFlagOutL && PingPongFlagOutR){
   			status = Dma_start((Dma_Obj *) &DMA0CH2);
   			status = Dma_start((Dma_Obj *) &DMA0CH3);
   			
   			status = Dma_start((Dma_Obj *) &DMA2CH0);
   			status = Dma_start((Dma_Obj *) &DMA2CH1);
   			
   			straightThroughFlag = 0;
    	}
    	else{
    		status = Dma_start((Dma_Obj *) &DMA0CH2);
   			status = Dma_start((Dma_Obj *) &DMA0CH3);
   			
   			status = Dma_start((Dma_Obj *) &DMA2CH2);
   			status = Dma_start((Dma_Obj *) &DMA2CH3);
   			
   			straightThroughFlag = 0;
    	}
    
   	}
*/
}


